@@ -1,5 +1,5 @@
 /* GNU ddrescue - Data recovery tool
-   Copyright (C) 2004-2022 Antonio Diaz Diaz.
+   Copyright (C) 2004-2025 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,35 +17,30 @@
 
 #define _FILE_OFFSET_BITS 64
 
-#include <algorithm>
 #include <cerrno>
-#include <climits>
-#include <cstdio>
 #include <cstring>
 #include <ctime>
-#include <string>
-#include <vector>
 #include <stdint.h>
 #include <unistd.h>
 
-#include "block.h"
+#include "mapfile.h"
 #include "mapbook.h"
 
 
-const char * format_time( const long t, const bool low_prec )
+const char * format_time( const long long t, const bool low_prec )
   {
   enum { buffers = 8, bufsize = 16 };
-  static char buffer[buffers][bufsize];	// circle of static buffers for printf
+  static char buffer[buffers][bufsize];	// circle of buffers for printf
   static int current = 0;
   if( t < 0 ) return "n/a";
   char * const buf = buffer[current++]; current %= buffers;
   const int s = t % 60;
   const int m = ( t / 60 ) % 60;
   const int h = ( t / 3600 ) % 24;
-  const long d = t / 86400;
+  const long long d = t / 86400;
   int len = 0;				// max len is 11 chars (10h 10m 10s)
 
-  if( d > 0 ) len = snprintf( buf, bufsize, "%ldd", d );
+  if( d > 0 ) len = snprintf( buf, bufsize, "%lldd", d );
   if( h > 0 && len >= 0 && len <= 7 )
     len += snprintf( buf + len, bufsize - len, "%s%dh", len ? ( (h < 10) ? "  " : " " ) : "", h );
   if( m > 0 && len >= 0 && len <= 7 )
@@ -105,8 +100,7 @@ int Genbook::check_all()
     check_block( b, copied_size, error_size );
     if( copied_size + error_size < b.size() &&			// EOF
         !truncate_vector( b.pos() + copied_size + error_size ) )
-      { final_msg( iname_, "EOF found below the size calculated from mapfile." );
-        return 1; }
+      { final_msg( iname_, early_eof_msg ); return 1; }
     if( !update_mapfile() ) return -2;
     }
   return 0;
@@ -126,7 +120,7 @@ void Genbook::show_status( const long long ipos, const char * const msg,
     }
 
   if( ipos >= 0 ) last_ipos = ipos;
-  const long t2 = std::time( 0 );
+  const long long t2 = std::time( 0 );
   if( t2 < t1 )					// clock jumped back
     {
     t0 -= std::min( t0, t1 - t2 );
@@ -148,13 +142,13 @@ void Genbook::show_status( const long long ipos, const char * const msg,
     std::printf( "   opos: %9sB,  run time: %11s,  average rate: %8sB/s\n",
                  format_num( last_ipos + offset() ), format_time( t1 - t0 ),
                  format_num( a_rate, 99999 ) );
-    if( msg && msg[0] )
+    if( msg && *msg )
       {
       const int len = std::strlen( msg ); std::printf( "\r%s", msg );
       for( int i = len; i < oldlen; ++i ) std::fputc( ' ', stdout );
       oldlen = len;
       }
-    std::fflush( stdout );
+    safe_fflush( stdout );
     }
   }
 
@@ -177,10 +171,10 @@ int Genbook::do_generate( const int odes )
   set_signals();
   if( verbosity >= 0 )
     {
-    std::fputs( "Press Ctrl-C to interrupt\n", stdout );
+    std::fputs( ctrlc_msg, stdout );
     if( mapfile_exists() )
       {
-      std::fputs( "Initial status (read from mapfile)\n", stdout );
+      std::fputs( initial_msg, stdout );
       std::printf( "rescued: %9sB,  generated: %9sB\n",
                    format_num( finished_size ), format_num( gensize ) );
       std::fputs( "Current status\n", stdout );
@@ -195,7 +189,7 @@ int Genbook::do_generate( const int odes )
     if( retval == -2 ) std::fputs( "\nMapfile error", stdout );
     else if( signaled ) std::fputs( "\nInterrupted by user", stdout );
     std::fputc( '\n', stdout );
-    std::fflush( stdout );
+    safe_fflush( stdout );
     }
   if( retval == -2 ) retval = 1;		// mapfile error
   else
