@@ -1,5 +1,5 @@
 /* GNU ddrescue - Data recovery tool
-   Copyright (C) 2013-2022 Antonio Diaz Diaz.
+   Copyright (C) 2013-2025 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,28 +17,26 @@
 
 #define _FILE_OFFSET_BITS 64
 
-#include <algorithm>
-#include <cstdio>
+#include <cerrno>
+#include <cstdlib>
 #include <cstring>
-#include <string>
-#include <vector>
 #include <sys/stat.h>
 
-#include "block.h"
+#include "mapfile.h"
 #include "loggers.h"
 
 
 namespace {
 
-const char * format_time_dhms( const long t )
+const char * format_time_dhms( const long long t )
   {
-  static char buf[32];
+  static char buf[64];
   const int s = t % 60;
   const int m = ( t / 60 ) % 60;
   const int h = ( t / 3600 ) % 24;
-  const long d = t / 86400;
+  const long long d = t / 86400;
 
-  if( d ) snprintf( buf, sizeof buf, "%ldd:%02dh:%02dm:%02ds", d, h, m, s );
+  if( d ) snprintf( buf, sizeof buf, "%lldd:%02dh:%02dm:%02ds", d, h, m, s );
   else if( h ) snprintf( buf, sizeof buf, "%dh:%02dm:%02ds", h, m, s );
   else if( m ) snprintf( buf, sizeof buf, "%dm:%02ds", m, s );
   else snprintf( buf, sizeof buf, "%ds", s );
@@ -55,7 +53,7 @@ Read_logger read_logger;
 
 void Logger::set_filename( const char * const name )
   {
-  if( !name || !name[0] ) return;			// ignore name
+  if( !name || !*name ) return;				// ignore name
   if( std::strcmp( name, "-" ) == 0 )
     { show_error( "I won't write log data to standard output." );
       std::exit( 1 ); }
@@ -101,7 +99,8 @@ bool Event_logger::echo_msg( const char * const msg )
   }
 
 
-bool Event_logger::print_msg( const long time, const char * const percent_rescued,
+bool Event_logger::print_msg( const long long time,
+                              const char * const percent_rescued,
                               const char * const msg )
   {
   if( f && !error &&
@@ -112,7 +111,8 @@ bool Event_logger::print_msg( const long time, const char * const percent_rescue
   }
 
 
-bool Event_logger::print_eor( const long time, const char * const percent_rescued,
+bool Event_logger::print_eor( const long long time,
+                              const char * const percent_rescued,
                               const long long current_pos,
                               const char * const current_status_name )
   {
@@ -139,7 +139,7 @@ bool Rate_logger::open_file()
   }
 
 
-bool Rate_logger::print_line( const long time, const long long ipos,
+bool Rate_logger::print_line( const long long time, const long long ipos,
                               const long long a_rate, const long long c_rate,
                               const unsigned long bad_areas,
                               const long long bad_size )
@@ -147,7 +147,7 @@ bool Rate_logger::print_line( const long time, const long long ipos,
   if( f && !error && time > last_time )
     {
     last_time = time;
-    if( std::fprintf( f, "%2ld  0x%08llX  %8lld  %8lld  %7lu  %8lld\n",
+    if( std::fprintf( f, "%2lld  0x%08llX  %8lld  %8lld  %7lu  %8lld\n",
                       time, ipos, c_rate, a_rate, bad_areas, bad_size ) < 0 )
       error = true;
     }
@@ -163,25 +163,32 @@ bool Read_logger::open_file()
     prev_is_msg = true;
     f = std::fopen( filename_, "w" );
     error = !f || !write_file_header( f, "Reads Logfile" ) ||
-            std::fputs( "#  Ipos       Size  Copied_size  Error_size\n", f ) == EOF;
+            std::fputs( "#  Ipos       Size  Copied_size  Error_size  Errno\n",
+                        f ) == EOF;
     }
   return !error;
   }
 
 
 bool Read_logger::print_line( const long long ipos, const long long size,
-                              const int copied_size, const int error_size )
+                              const int copied_size, const int error_size,
+                              const int errcode )
   {
-  if( f && !error &&
-      std::fprintf( f, "0x%08llX	%lld	%d	%d\n",
-                    ipos, size, copied_size, error_size ) < 0 )
-    error = true;
+  if( f && !error )
+    {
+    if( errcode <= 0 || errcode == EIO )
+      { if( std::fprintf( f, "0x%08llX	%lld	%d	%d\n", ipos,
+                          size, copied_size, error_size ) < 0 ) error = true; }
+    else
+      { if( std::fprintf( f, "0x%08llX	%lld	%d	%d	%d\n", ipos,
+                 size, copied_size, error_size, errcode ) < 0 ) error = true; }
+    }
   prev_is_msg = false;
   return !error;
   }
 
 
-bool Read_logger::print_msg( const long time, const char * const msg )
+bool Read_logger::print_msg( const long long time, const char * const msg )
   {
   if( f && !error &&
       std::fprintf( f, "%s# %s  %s\n", prev_is_msg ? "" : "\n",
@@ -192,7 +199,7 @@ bool Read_logger::print_msg( const long time, const char * const msg )
   }
 
 
-bool Read_logger::print_time( const long time )
+bool Read_logger::print_time( const long long time )
   {
   if( f && !error && time > 0 &&
       std::fprintf( f, "# %s\n", format_time_dhms( time ) ) < 0 )
