@@ -1,5 +1,5 @@
 /* GNU ddrescue - Data recovery tool
-   Copyright (C) 2013-2022 Antonio Diaz Diaz.
+   Copyright (C) 2013-2025 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +15,8 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+// requires '#include <cstdio>' for 'FILE *'
+
 class Logger
   {
 protected:
@@ -37,9 +39,9 @@ class Event_logger : public Logger
 public:
   bool open_file();
   bool echo_msg( const char * const msg );
-  bool print_msg( const long time, const char * const percent_rescued,
+  bool print_msg( const long long time, const char * const percent_rescued,
                   const char * const msg );
-  bool print_eor( const long time, const char * const percent_rescued,
+  bool print_eor( const long long time, const char * const percent_rescued,
                   const long long current_pos,
                   const char * const current_status_name );
   };
@@ -49,11 +51,11 @@ extern Event_logger event_logger;
 
 class Rate_logger : public Logger
   {
-  long last_time;
+  long long last_time;
 public:
   Rate_logger() : last_time( -1 ) {}
   bool open_file();
-  bool print_line( const long time, const long long ipos,
+  bool print_line( const long long time, const long long ipos,
                    const long long a_rate, const long long c_rate,
                    const unsigned long bad_areas,
                    const long long bad_size );
@@ -69,9 +71,10 @@ public:
   Read_logger() : prev_is_msg( true ) {}
   bool open_file();
   bool print_line( const long long ipos, const long long size,
-                   const int copied_size, const int error_size );
-  bool print_msg( const long time, const char * const msg );
-  bool print_time( const long time );
+                   const int copied_size, const int error_size,
+                   const int errcode );
+  bool print_msg( const long long time, const char * const msg );
+  bool print_time( const long long time );
   };
 
 extern Read_logger read_logger;
