@@ -1,20 +1,20 @@
-/*  Rational - Rational number class with overflow detection
-    Copyright (C) 2005-2020 Antonio Diaz Diaz.
+/* Rational - Rational number class with overflow detection
+   Copyright (C) 2005-2025 Antonio Diaz Diaz.
 
-    This library is free software. Redistribution and use in source and
-    binary forms, with or without modification, are permitted provided
-    that the following conditions are met:
+   This library is free software. Redistribution and use in source and
+   binary forms, with or without modification, are permitted provided
+   that the following conditions are met:
 
-    1. Redistributions of source code must retain the above copyright
-    notice, this list of conditions, and the following disclaimer.
+   1. Redistributions of source code must retain the above copyright
+   notice, this list of conditions, and the following disclaimer.
 
-    2. Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions, and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
+   2. Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions, and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
 
 #include <algorithm>
@@ -26,10 +26,10 @@
 #include "rational.h"
 
 #ifndef LLONG_MAX
-#define LLONG_MAX  0x7FFFFFFFFFFFFFFFLL
+#define LLONG_MAX 0x7FFFFFFFFFFFFFFFLL
 #endif
 #ifndef LLONG_MIN
-#define LLONG_MIN  (-LLONG_MAX - 1LL)
+#define LLONG_MIN (-LLONG_MAX - 1LL)
 #endif
 #ifndef ULLONG_MAX
 #define ULLONG_MAX 0xFFFFFFFFFFFFFFFFULL
@@ -69,16 +69,12 @@ void Rational::normalize( long long n, long long d )
   {
   if( d == 0 ) { num = overflow_value( n ); den = 0; return; }  // set error
   if( n == 0 ) { num = 0; den = 1; return; }
-  if( d != 1 )
-    {
-    const long long tmp = gcd( n, d );
-    n /= tmp; d /= tmp;
-    }
+  if( d < 0 ) { n = -n; d = -d; }			// just in case
+  if( d != 1 ) { const long long tmp = gcd( n, d ); n /= tmp; d /= tmp; }
 
-  if( n <= INT_MAX && n >= -INT_MAX && d <= INT_MAX && d >= -INT_MAX )
-    { if( d >= 0 ) { num = n; den = d; } else { num = -n; den = -d; } }
-  else
-    { num = overflow_value( n, d < 0 ); den = 0; }
+  if( n <= INT_MAX && n >= -INT_MAX && d <= INT_MAX && d > 0 )
+    { num = n; den = d; return; }	// n / d is in range within precision
+  num = overflow_value( n, d < 0 ); den = 0;
   }
 
 
@@ -104,7 +100,7 @@ void Rational::normalize()
 
 Rational Rational::inverse() const
   {
-  if( den <= 0 ) return *this;			// no op on error
+  if( den <= 0 ) return *this;			// no-op on error
   Rational tmp;
   if( num > 0 ) { tmp.num = den; tmp.den = num; }
   else if( num < 0 ) { tmp.num = -den; tmp.den = -num; }
@@ -115,7 +111,7 @@ Rational Rational::inverse() const
 
 Rational & Rational::operator+=( const Rational & r )
   {
-  if( den <= 0 ) return *this;			// no op on error
+  if( den <= 0 ) return *this;			// no-op on error
   if( r.den <= 0 ) { num = r.num; den = 0; return *this; }	// set error
 
   long long new_den = den; new_den *= r.den;
@@ -128,7 +124,7 @@ Rational & Rational::operator+=( const Rational & r )
 
 Rational & Rational::operator*=( const Rational & r )
   {
-  if( den <= 0 ) return *this;			// no op on error
+  if( den <= 0 ) return *this;			// no-op on error
   if( r.den <= 0 ) { num = r.num; den = 0; return *this; }	// set error
 
   long long new_num = num; new_num *= r.num;
@@ -149,11 +145,11 @@ int Rational::round() const
   }
 
 
-// Recognized formats: 123 123/456 123.456 .123 12% 12/3% 12.3% .12%
-// Values may be preceded by an optional '+' or '-' sign.
-// Returns the number of chars read from 's', or 0 if input is invalid.
-// In case of invalid input, the Rational is not changed.
-//
+/* Recognized formats: 123 123/456 123.456 .123 12% 12/3% 12.3% .12%
+   Values may be preceded by an optional '+' or '-' sign.
+   Return the number of chars read from 's', or 0 if input is invalid.
+   In case of invalid input, the Rational is not changed.
+*/
 int Rational::parse( const char * const s )
   {
   if( !s || !s[0] ) return 0;
@@ -166,7 +162,7 @@ int Rational::parse( const char * const s )
   else if( s[c] == '-' ) { ++c; minus = true; }
   if( !std::isdigit( s[c] ) && s[c] != '.' ) return 0;
 
-  while( std::isdigit( s[c] ) )
+  while( std::isdigit( s[c] ) )		// integer part or numerator
     {
     if( ( LLONG_MAX - (s[c] - '0') ) / 10 < n ) return 0;
     n = (n * 10) + (s[c] - '0'); ++c;
@@ -209,13 +205,15 @@ int Rational::parse( const char * const s )
   }
 
 
-// Returns a string representing the value 'num/den' in decimal point
-// format with 'prec' decimals.
-// 'iwidth' is the minimum width of the integer part, prefixed with
-// spaces if needed.
-// If 'prec' is negative, only the decimals needed are produced.
-//
-std::string Rational::to_decimal( const unsigned iwidth, int prec ) const
+/* Return a string representing the value 'num/den' in decimal point format
+   with 'prec' decimals.
+   'iwidth' is the minimum width of the integer part, prefixed with spaces
+   if needed.
+   If 'prec' is negative, produce only the decimals needed.
+   If 'rounding', round up the last digit if the next one would be >= 5.
+*/
+std::string Rational::to_decimal( const unsigned iwidth, int prec,
+                                  const bool rounding ) const
   {
   if( den <= 0 ) return overflow_string( num );
 
@@ -224,7 +222,7 @@ std::string Rational::to_decimal( const unsigned iwidth, int prec ) const
   const bool truncate = ( prec < 0 );
   if( prec < 0 ) prec = -prec;
 
-  do { s += '0' + ( ipart % 10 ); ipart /= 10; } while( ipart > 0 );
+  do { s += ( ipart % 10 ) + '0'; ipart /= 10; } while( ipart > 0 );
   if( num < 0 ) s += '-';
   if( iwidth > s.size() ) s.append( iwidth - s.size(), ' ' );
   std::reverse( s.begin(), s.end() );
@@ -233,16 +231,30 @@ std::string Rational::to_decimal( const unsigned iwidth, int prec ) const
     {
     s += '.';
     while( prec > 0 && ( rest > 0 || !truncate ) )
-      { rest *= 10; s += '0' + ( rest / den ); rest %= den; --prec; }
+      { rest *= 10; s += ( rest / den ) + '0'; rest %= den; --prec; }
     }
+  if( rounding && rest * 2 >= den )		// round last decimal up
+    for( int j = s.size() - 1; j >= 0; --j )
+      {
+      if( s[j] == '.' ) continue;
+      if( s[j] >= '0' && s[j] < '9' ) { ++s[j]; break; }
+      if( s[j] == '9' ) s[j] = '0';
+      if( j > 0 && s[j-1] == '.' ) continue;
+      if( j > 0 && s[j-1] == ' ' ) { s[j-1] = '1'; break; }
+      if( j > 1 && s[j-2] == ' ' && s[j-1] == '-' )
+        { s[j-2] = '-'; s[j-1] = '1'; break; }
+      // no prev digit, prepend '1' to the first digit
+      if( j == 0 || s[j-1] < '0' || s[j-1] > '9' )
+        { s.insert( s.begin() + j, '1' ); break; }
+      }
   return s;
   }
 
 
-// Returns a string representing the value 'num/den' in fractional form.
-// 'width' is the minimum width to be produced, prefixed with spaces if
-// needed.
-//
+/* Return a string representing the value 'num/den' in fractional form.
+   'width' is the minimum width to be produced, prefixed with spaces if
+   needed. If den == 1, print num alone.
+*/
 std::string Rational::to_fraction( const unsigned width ) const
   {
   if( den <= 0 ) return overflow_string( num );
@@ -250,9 +262,9 @@ std::string Rational::to_fraction( const unsigned width ) const
   std::string s;
   int n = std::abs( num ), d = den;
 
-  do { s += '0' + ( d % 10 ); d /= 10; } while( d > 0 );
-  s += '/';
-  do { s += '0' + ( n % 10 ); n /= 10; } while( n > 0 );
+  if( d > 1 )
+    { do { s += ( d % 10 ) + '0'; d /= 10; } while( d > 0 ); s += '/'; }
+  do { s += ( n % 10 ) + '0'; n /= 10; } while( n > 0 );
   if( num < 0 ) s += '-';
   if( width > s.size() ) s.append( width - s.size(), ' ' );
   std::reverse( s.begin(), s.end() );
