@@ -1,5 +1,5 @@
 /* GNU ddrescue - Data recovery tool
-   Copyright (C) 2004-2022 Antonio Diaz Diaz.
+   Copyright (C) 2004-2025 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,23 +17,18 @@
 
 #define _FILE_OFFSET_BITS 64
 
-#include <algorithm>
 #include <cctype>
 #include <cerrno>
-#include <climits>
-#include <cstdio>
 #include <cstring>
 #include <ctime>
-#include <string>
-#include <vector>
 #include <stdint.h>
 #include <unistd.h>
 #include <sys/stat.h>
 
-#include "rational.h"
-#include "block.h"
+#include "mapfile.h"
 #include "loggers.h"
 #include "mapbook.h"
+#include "rational.h"
 #include "rescuebook.h"
 
 
@@ -114,38 +109,44 @@ bool Rescuebook::extend_outfile_size()
 */
 int Rescuebook::copy_block( const Block & b, int & copied_size, int & error_size )
   {
-  if( b.size() <= 0 ) internal_error( "bad size copying a Block." );
-  if( !test_domain || test_domain->includes( b ) )
+  if( b.size() <= 0 ) internal_error( "bad block size in copy_block." );
+  int saved_errno;				// for read_logger
+  if( !test_domainp || test_domainp->includes( b ) )
     {
-    if( o_direct_in )
+    if( o_direct_in || bsd_buf )  // read whole sectors, write requested part
       {
       const int pre = b.pos() % hardbs();
-      const int disp = b.end() % hardbs();
-      const int post = ( disp > 0 ) ? hardbs() - disp : 0;
+      const int rest = b.end() % hardbs();
+      const int post = ( rest > 0 ) ? hardbs() - rest : 0;
       const int size = pre + b.size() + post;
       if( size > iobuf_size() )
         internal_error( "(size > iobuf_size) copying a Block." );
       copied_size = readblockp( ides_, iobuf(), size, b.pos() - pre );
+      adjust_copied_size( copied_size );	// discard bad_sector_data
       copied_size -= std::min( pre, copied_size );
       if( copied_size > b.size() ) copied_size = b.size();
       if( pre > 0 && copied_size > 0 )
         std::memmove( iobuf(), iobuf() + pre, copied_size );
       }
     else copied_size = readblockp( ides_, iobuf(), b.size(), b.pos() );
+    saved_errno = errno;
     error_size = errno ? b.size() - copied_size : 0;
     if( copied_size <= 0 ) switch( errno )
       {
       case EACCES: case EBADF: case EBUSY: case EISDIR: case ENOBUFS:
       case ENODEV: case ENOENT: case ENOMEM: case ENOSYS: case ENXIO:
       case EPERM: case ESPIPE:
+        for( unsigned i = 0; i < errno_vector.size(); ++i )
+          if( errno_vector[i] == errno ) goto non_fatal;
         final_msg( iname_, "Fatal error reading the input file", errno );
         return 1;
       case EINVAL:
         final_msg( iname_, "Unaligned read error. Is sector size correct?" );
         return 1;
       }
+non_fatal: ;
     }
-  else { copied_size = 0; error_size = b.size(); }
+  else { copied_size = 0; error_size = b.size(); saved_errno = EIO; }
 
   if( copied_size > 0 )
     {
@@ -156,27 +157,31 @@ int Rescuebook::copy_block( const Block & b, int & copied_size, int & error_size
       const long long end = pos + copied_size;
       if( end > sparse_size ) sparse_size = end;
       }
-    else if( writeblockp( odes_, iobuf(), copied_size, pos ) != copied_size ||
-             ( synchronous_ && fsync( odes_ ) != 0 && errno != EINVAL ) )
-      { final_msg( oname_, "Write error", errno ); return 1; }
+    else
+      if( !compare_before_write ||
+          readblockp( odes_, iobuf2(), copied_size, pos ) != copied_size ||
+          std::memcmp( iobuf(), iobuf2(), copied_size ) != 0 )
+        if( writeblockp( odes_, iobuf(), copied_size, pos ) != copied_size ||
+              ( synchronous_ && fsync( odes_ ) != 0 && errno != EINVAL ) )
+          { final_msg( oname_, wr_err_msg, errno ); return 1; }
     }
   else iobuf_ipos = -1;
 
-  read_logger.print_line( b.pos(), b.size(), copied_size, error_size );
+  read_logger.print_line( b.pos(), b.size(), copied_size, error_size, saved_errno );
 
-  if( verify_on_error )
+  if( check_on_error )
     {
     if( copied_size >= hardbs() && b.pos() % hardbs() == 0 )
-      { voe_ipos = b.pos(); std::memcpy( voe_buf, iobuf(), hardbs() ); }
+      { coe_ipos = b.pos(); std::memcpy( coe_buf, iobuf(), hardbs() ); }
     if( error_size > 0 )
       {
-      if( voe_ipos >= 0 )
+      if( coe_ipos >= 0 )
         {
-        const int size = readblockp( ides_, iobuf_aux(), hardbs(), voe_ipos );
+        const int size = readblockp( ides_, iobuf2(), hardbs(), coe_ipos );
         if( size != hardbs() )
           { final_msg( iname_, "Input file no longer returns data", errno );
             e_code |= 8; }
-        else if( std::memcmp( voe_buf, iobuf_aux(), hardbs() ) != 0 )
+        else if( std::memcmp( coe_buf, iobuf2(), hardbs() ) != 0 )
           { final_msg( iname_, "Input file returns inconsistent data." );
             e_code |= 8; }
         }
@@ -227,7 +232,7 @@ int Rescuebook::copy_and_update( const Block & b, int & copied_size,
       {
       show_status( -1, "Paused", true );
       sleep( pause_on_pass );
-      const long t2 = std::time( 0 );
+      const long long t2 = std::time( 0 );
       if( t1 < t2 ) t1 = t2;			// clock may have jumped back
       ts = std::min( ts + pause_on_pass, t2 );	// avoid spurious timeout
       }
@@ -247,8 +252,7 @@ int Rescuebook::copy_and_update( const Block & b, int & copied_size,
       {
       if( complete_only ) truncate_domain( b.pos() + copied_size + error_size );
       else if( !truncate_vector( b.pos() + copied_size + error_size ) )
-        { final_msg( iname_, "EOF found below the size calculated from mapfile." );
-          retval = 1; }
+        { final_msg( iname_, early_eof_msg ); retval = 1; }
       initialize_sizes();
       }
     if( copied_size > 0 )
@@ -263,7 +267,7 @@ int Rescuebook::copy_and_update( const Block & b, int & copied_size,
       change_chunk_status( Block( b.pos() + copied_size, error_size ), st2 );
       struct stat istat;
       if( stat( iname_, &istat ) != 0 )
-        { final_msg( iname_, "Input file disappeared", errno ); retval = 1; }
+        { final_msg( iname_, disap_msg, errno ); retval = 1; }
       }
     }
   return retval;
@@ -312,7 +316,7 @@ int Rescuebook::fcopy_non_tried( const char * const msg, const int pass,
   long long pos = 0;
   long long eskip_size = skipbs;	// size to skip on error if skipbs > 0
   long long sskip_size = 0;		// size to skip on slow if skipbs > 0
-  const bool after_finished = (pass == 3 || pass == 4 );
+  const bool after_finished = pass == 3 || pass == 4;
   bool block_found = false;
   bool block_processed = false;
 
@@ -387,7 +391,7 @@ int Rescuebook::rcopy_non_tried( const char * const msg, const int pass,
   long long end = LLONG_MAX;
   long long eskip_size = skipbs;	// size to skip on error if skipbs > 0
   long long sskip_size = 0;		// size to skip on slow if skipbs > 0
-  const bool before_finished = (pass == 3 || pass == 4 );
+  const bool before_finished = pass == 3 || pass == 4;
   bool block_found = false;
   bool block_processed = false;
 
@@ -489,8 +493,9 @@ int Rescuebook::trim_errors()
                                           trimming, 1, true );
       if( retval ) return retval;
       update_rates();
-      if( error_size > 0 )
-        { error_found = true; if( pause_on_error > 0 ) do_pause_on_error(); }
+      if( error_size > 0 ) { error_found = true;
+        if( reopen_on_error && !reopen_infile() ) return 1;
+        if( pause_on_error > 0 ) do_pause_on_error(); }
       if( !update_mapfile( odes_ ) ) return -2;
       }
     error_found = rbad;
@@ -505,8 +510,9 @@ int Rescuebook::trim_errors()
                                           trimming, 1, false );
       if( retval ) return retval;
       update_rates();
-      if( error_size > 0 )
-        { error_found = true; if( pause_on_error > 0 ) do_pause_on_error(); }
+      if( error_size > 0 ) { error_found = true;
+        if( reopen_on_error && !reopen_infile() ) return 1;
+        if( pause_on_error > 0 ) do_pause_on_error(); }
       if( !update_mapfile( odes_ ) ) return -2;
       }
     if( pos < end )
@@ -551,7 +557,9 @@ int Rescuebook::scrape_errors()
                                           scraping, 1, true );
       if( retval ) return retval;
       update_rates();
-      if( error_size > 0 && pause_on_error > 0 ) do_pause_on_error();
+      if( error_size > 0 )
+        { if( reopen_on_error && !reopen_infile() ) return 1;
+          if( pause_on_error > 0 ) do_pause_on_error(); }
       if( !update_mapfile( odes_ ) ) return -2;
       }
     }
@@ -618,7 +626,9 @@ int Rescuebook::fcopy_errors( const char * const msg, const int pass,
                                         retrying, pass, true );
     if( retval ) return retval;
     update_rates();
-    if( error_size > 0 && pause_on_error > 0 ) do_pause_on_error();
+    if( error_size > 0 )
+      { if( reopen_on_error && !reopen_infile() ) return 1;
+        if( pause_on_error > 0 ) do_pause_on_error(); }
     if( !update_mapfile( odes_ ) ) return -2;
     }
   if( !block_found ) return 0;
@@ -655,7 +665,9 @@ int Rescuebook::rcopy_errors( const char * const msg, const int pass,
                                         retrying, pass, false );
     if( retval ) return retval;
     update_rates();
-    if( error_size > 0 && pause_on_error > 0 ) do_pause_on_error();
+    if( error_size > 0 )
+      { if( reopen_on_error && !reopen_infile() ) return 1;
+        if( pause_on_error > 0 ) do_pause_on_error(); }
     if( !update_mapfile( odes_ ) ) return -2;
     }
   if( !block_found ) return 0;
@@ -681,12 +693,12 @@ bool Rescuebook::update_rates( const bool force )
       }
     }
 
-  long t2 = std::time( 0 );
+  long long t2 = std::time( 0 );
   if( max_read_rate > 0 && finished_size - last_size > max_read_rate && t2 == t1 )
     { sleep( 1 ); t2 = std::time( 0 ); }
   if( t2 < t1 )					// clock jumped back
     {
-    const long delta = std::min( t0 - 1, t1 - t2 );
+    const long long delta = std::min( t0 - 1, t1 - t2 );
     t0 -= delta;
     ts -= delta;
     t1 = t2;
@@ -697,7 +709,7 @@ bool Rescuebook::update_rates( const bool force )
     {
     if( tp > 0 )
       {
-      const long delta = std::min( t0 - 1, (long)tp.round() );
+      const long long delta = std::min( t0 - 1, (long long)tp.round() );
       t0 -= delta;
       t1 -= delta;
       ts -= delta;
@@ -772,17 +784,17 @@ void Rescuebook::show_status( const long long ipos, const char * const msg,
           }
         std::fputc( '\n', stdout );
         }
-      std::printf( "     ipos: %9sB, non-trimmed: %9sB,  current rate: %8sB/s\n",
+      std::printf( "     ipos: %9sB, non-trimmed: %9sB,   current rate: %8sB/s\n",
                    format_num( last_ipos ), format_num( non_trimmed_size ),
                    format_num( c_rate, 99999 ) );
-      std::printf( "     opos: %9sB, non-scraped: %9sB,  average rate: %8sB/s\n",
+      std::printf( "     opos: %9sB, non-scraped: %9sB,   average rate: %8sB/s\n",
                    format_num( last_ipos + offset() ),
                    format_num( non_scraped_size ), format_num( a_rate, 99999 ) );
-      std::printf( "non-tried: %9sB,  bad-sector: %9sB,    error rate: %8sB/s\n",
+      std::printf( "non-tried: %9sB,  bad-sector: %9sB,     error rate: %8sB/s\n",
                    format_num( non_tried_size ), format_num( bad_size ),
                    format_num( error_rate, 99999 ) );
-      std::printf( "  rescued: %9sB,   bad areas: %8lu,        run time: %11s\n",
-                   format_num( finished_size ), bad_areas,
+      std::printf( "  rescued: %9sB,   bad areas:%11s,       run time: %11s\n",
+                   format_num( finished_size ), format_num3( bad_areas ),
                    format_time( t1 - t0 ) );
       if( first_post ) sliding_avg.reset();
       else sliding_avg.add_term( c_rate );
@@ -791,20 +803,21 @@ void Rescuebook::show_status( const long long ipos, const char * const msg,
         std::min( std::min( (long long)LONG_MAX, 315359999968464000LL ),
                   ( non_tried_size + non_trimmed_size + non_scraped_size +
                     ( max_retries ? bad_size : 0 ) + s_rate - 1 ) / s_rate );
-      std::printf( "pct rescued:  %s, read errors:%9lu,  remaining time: %11s\n",
-                   percent_rescued(), read_errors,
+      std::printf( "pct rescued:  %s, read errors:%11s, remaining time: %11s\n",
+                   percent_rescued(), format_num3( read_errors ),
                    format_time( remaining_time, remaining_time >= 180 ) );
-      if( min_read_rate >= -1 ) std::printf( " slow reads:%9lu,", slow_reads );
+      if( min_read_rate >= -1 )
+        std::printf( "slow reads:%10s,", format_num3( slow_reads ) );
       else std::fputs( "                      ", stdout );
-      std::printf( "        time since last successful read: %11s\n",
+      std::printf( "         time since last successful read: %11s\n",
                    format_time( ( ts > t0 ) ? t1 - ts : -1 ) );
-      if( msg && msg[0] && !errors_or_timeout() )
+      if( msg && *msg && !errors_or_timeout() )
         {
         const int len = std::strlen( msg ); std::printf( "\r%s", msg );
         for( int i = len; i < oldlen; ++i ) std::fputc( ' ', stdout );
         oldlen = len;
         }
-      std::fflush( stdout );
+      safe_fflush( stdout );
       }
     rate_logger.print_line( t1 - t0, last_ipos, a_rate, c_rate, bad_areas,
                             bad_size );
@@ -824,22 +837,13 @@ Rescuebook::Rescuebook( const long long offset, const long long insize,
                         const int hardbs, const bool synchronous )
   : Mapbook( offset, insize, dom, mb_opts, mapname, cluster, hardbs,
              rb_opts.complete_only, true ),
-    Rb_options( rb_opts ),
-    error_rate( 0 ),
-    error_sum( 0 ),
-    sparse_size( sparse ? 0 : -1 ),
-    non_tried_size( 0 ),
-    non_trimmed_size( 0 ),
-    non_scraped_size( 0 ),
-    bad_size( 0 ),
-    finished_size( 0 ),
-    test_domain( test_dom ),
-    iname_( iname ), oname_( oname ),
-    read_errors( 0 ),
-    slow_reads( 0 ),
-    e_code( 0 ),
+    Rb_options( rb_opts ), error_rate( 0 ), error_sum( 0 ),
+    sparse_size( sparse ? 0 : -1 ), non_tried_size( 0 ),
+    non_trimmed_size( 0 ), non_scraped_size( 0 ), bad_size( 0 ),
+    finished_size( 0 ), test_domainp( test_dom ), iname_( iname ),
+    oname_( oname ), read_errors( 0 ), slow_reads( 0 ), e_code( 0 ),
     synchronous_( synchronous ),
-    voe_ipos( -1 ), voe_buf( new uint8_t[hardbs] ),
+    coe_ipos( -1 ), coe_buf( new uint8_t[hardbs] ), bsd_buf( 0 ),
     a_rate( 0 ), c_rate( 0 ), first_size( 0 ), last_size( 0 ),
     iobuf_ipos( -1 ), last_ipos( 0 ), t0( 0 ), t1( 0 ), ts( 0 ), tp( 0 ),
     oldlen( 0 ), rates_updated( false ), current_slow( false ),
@@ -882,6 +886,16 @@ Rescuebook::Rescuebook( const long long offset, const long long insize,
   }
 
 
+bool Rescuebook::read_bad_sector_data( const int fd )
+  {
+  if( bsd_buf ) return false;
+  uint8_t * const p = new uint8_t[hardbs()];
+  bsd_buf = p;
+  const int rd = readblock( fd, p, hardbs() );
+  return rd == hardbs() && errno == 0;
+  }
+
+
 // Return values: 0 OK, != 0 error.
 //
 int Rescuebook::do_rescue( const int ides, const int odes )
@@ -890,26 +904,28 @@ int Rescuebook::do_rescue( const int ides, const int odes )
   set_signals();
   if( verbosity >= 0 )
     {
-    std::fputs( "Press Ctrl-C to interrupt\n", stdout );
+    std::fputs( ctrlc_msg, stdout );
     if( mapfile_exists() )
       {
-      std::fputs( "Initial status (read from mapfile)\n", stdout );
+      std::fputs( initial_msg, stdout );
       if( verbosity >= 3 )
         {
-        std::printf( "current position: %9sB,     current sector: %7lld\n",
-                     format_num( current_pos() ), current_pos() / hardbs() );
+        std::printf( "current position: %9sB,     current sector: %7s\n",
+                     format_num( current_pos() ),
+                     format_num3( current_pos() / hardbs() ) );
         if( sblocks() )
           std::printf( " last block size: %9sB\n",
                        format_num( sblock( sblocks() - 1 ).size() ) );
         }
       if( domain().pos() > 0 || domain().end() < mapfile_insize() )
-        std::printf( "(sizes limited to domain from %s B to %s B of %s B)\n",
-                     format_num3( domain().pos() ), format_num3( domain().end() ),
-                     format_num3( mapfile_insize() ) );
-      std::printf( "rescued: %sB, tried: %sB, bad-sector: %sB, bad areas: %lu\n\n",
+        std::printf( "(sizes limited to domain from %sB to %sB of %sB)\n",
+                     format_num3p( domain().pos(), true ),
+                     format_num3p( domain().end(), true ),
+                     format_num3p( mapfile_insize(), true ) );
+      std::printf( "rescued: %sB, tried: %sB, bad-sector: %sB, bad areas: %s\n\n",
                    format_num( finished_size ),
                    format_num( non_trimmed_size + non_scraped_size + bad_size ),
-                   format_num( bad_size ), bad_areas );
+                   format_num( bad_size ), format_num3( bad_areas ) );
       std::fputs( "Current status\n", stdout );
       }
     }
@@ -923,6 +939,7 @@ int Rescuebook::do_rescue( const int ides, const int odes )
     retval = scrape_errors();
   if( retval == 0 && bad_size && max_retries != 0 && !errors_or_timeout() )
     retval = copy_errors();
+  fsync( odes_ );	// prevent early exit if kernel caches writes
   if( !rates_updated ) update_rates( true );	// force update of e_code
   show_status( -1, retval ? 0 : "\nFinished", true );
 
@@ -949,7 +966,7 @@ int Rescuebook::do_rescue( const int ides, const int odes )
       if( e_code & 32 ) event_logger.echo_msg( "Too many slow reads" );
       }
     if( verbosity >= 0 )
-      { std::fputc( '\n', stdout ); std::fflush( stdout ); }
+      { std::fputc( '\n', stdout ); safe_fflush( stdout ); }
     }
   if( retval == -2 ) retval = 1;		// mapfile error
   else
@@ -963,7 +980,7 @@ int Rescuebook::do_rescue( const int ides, const int odes )
     compact_sblock_vector();
     if( !update_mapfile( odes_, true ) && retval == 0 ) retval = 1;
     }
-  if( final_msg().size() ) show_error( final_msg().c_str(), final_errno() );
+  if( final_msg().size() ) show_final_msg( final_msg().c_str(), final_errno() );
   if( close( odes_ ) != 0 )
     { show_file_error( oname_, "Error closing outfile", errno );
       if( retval == 0 ) retval = 1; }
@@ -971,13 +988,13 @@ int Rescuebook::do_rescue( const int ides, const int odes )
                           status_name( current_status() ) );
   if( !event_logger.close_file() )
     show_file_error( event_logger.filename(),
-                     "warning: Error closing the events logging file." );
+                     "warning: error closing the events logging file." );
   if( !rate_logger.close_file() )
     show_file_error( rate_logger.filename(),
-                     "warning: Error closing the rates logging file." );
+                     "warning: error closing the rates logging file." );
   if( !read_logger.close_file() )
     show_file_error( read_logger.filename(),
-                     "warning: Error closing the reads logging file." );
+                     "warning: error closing the reads logging file." );
   if( retval ) return retval;		// errors have priority over signals
   if( signaled ) return signaled_exit();
   return 0;
