@@ -1,5 +1,5 @@
 /* GNU ddrescue - Data recovery tool
-   Copyright (C) 2004-2022 Antonio Diaz Diaz.
+   Copyright (C) 2004-2025 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
 
 class Sliding_average		// Calculates the average of the last N terms
   {
-  unsigned index;
-  std::vector<long long> data;
+  unsigned index;		// either index or data.size() contain N
+  std::vector< long long > data;
 
 public:
   explicit Sliding_average( const unsigned terms ) : index( terms )
@@ -54,6 +54,7 @@ struct Rb_options
   long long min_read_rate;	// -2 = not set, -1 = reset
   long long skipbs;		// initial size to skip on read error
   long long max_skipbs;		// maximum size to skip on read error
+  std::vector< int > errno_vector;	// treat these values as non-fatal
   unsigned long max_bad_areas;
   unsigned long max_read_errors;
   unsigned long max_slow_reads;
@@ -65,6 +66,7 @@ struct Rb_options
   int pause_on_pass;
   int preview_lines;		// preview lines to show. 0 = disable
   int timeout;
+  bool compare_before_write;
   bool complete_only;
   bool new_bad_areas_only;
   bool noscrape;
@@ -78,7 +80,7 @@ struct Rb_options
   bool sparse;
   bool try_again;
   bool unidirectional;
-  bool verify_on_error;
+  bool check_on_error;
 
   Rb_options()
     : max_max_skipbs( 1LL << 60 ), max_error_rate( -1 ), min_outfile_size( -1 ),
@@ -87,11 +89,12 @@ struct Rb_options
       max_read_errors( ULONG_MAX ), max_slow_reads( ULONG_MAX ),
       cpass_bitset( 31 ), delay_slow( 30 ), max_retries( 0 ), o_direct_in( 0 ),
       pause_on_error( 0 ), pause_on_pass( 0 ), preview_lines( 0 ),
-      timeout( -1 ), complete_only( false ), new_bad_areas_only( false ),
-      noscrape( false ), notrim( false ), reopen_on_error( false ),
-      reset_slow( false ), retrim( false ), reverse( false ),
-      same_file( false ), simulated_poe( false ), sparse( false ),
-      try_again( false ), unidirectional( false ), verify_on_error( false )
+      timeout( -1 ), compare_before_write( false ), complete_only( false ),
+      new_bad_areas_only( false ), noscrape( false ), notrim( false ),
+      reopen_on_error( false ), reset_slow( false ), retrim( false ),
+      reverse( false ), same_file( false ), simulated_poe( false ),
+      sparse( false ), try_again( false ), unidirectional( false ),
+      check_on_error( false )
       {}
 
   bool operator==( const Rb_options & o ) const
@@ -100,6 +103,7 @@ struct Rb_options
                max_read_rate == o.max_read_rate &&
                min_read_rate == o.min_read_rate &&
                skipbs == o.skipbs && max_skipbs == o.max_skipbs &&
+               errno_vector == o.errno_vector &&
                max_bad_areas == o.max_bad_areas &&
                max_read_errors == o.max_read_errors &&
                max_slow_reads == o.max_slow_reads &&
@@ -110,6 +114,7 @@ struct Rb_options
                pause_on_error == o.pause_on_error &&
                pause_on_pass == o.pause_on_pass &&
                preview_lines == o.preview_lines && timeout == o.timeout &&
+               compare_before_write == o.compare_before_write &&
                complete_only == o.complete_only &&
                new_bad_areas_only == o.new_bad_areas_only &&
                noscrape == o.noscrape && notrim == o.notrim &&
@@ -120,7 +125,7 @@ struct Rb_options
                simulated_poe == o.simulated_poe &&
                sparse == o.sparse && try_again == o.try_again &&
                unidirectional == o.unidirectional &&
-               verify_on_error == o.verify_on_error ); }
+               check_on_error == o.check_on_error ); }
   bool operator!=( const Rb_options & o ) const
     { return !( *this == o ); }
   };
@@ -132,7 +137,7 @@ class Rescuebook : public Mapbook, public Rb_options
   long long sparse_size;		// end position of pending writes
   long long non_tried_size, non_trimmed_size, non_scraped_size;
   long long bad_size, finished_size;
-  const Domain * const test_domain;	// good/bad map for test mode
+  const Domain * const test_domainp;	// good/bad map for test mode
   const char * const iname_, * const oname_;
   unsigned long bad_areas;		// bad areas found so far
   unsigned long read_errors, slow_reads;
@@ -142,13 +147,14 @@ class Rescuebook : public Mapbook, public Rb_options
 					// 8 other (explained in final_msg),
 					// 16 read_errors, 32 slow_reads
   const bool synchronous_;
-  long long voe_ipos;			// pos of last good sector read, or -1
-  uint8_t * const voe_buf;		// copy of last good sector read
+  long long coe_ipos;			// pos of last good sector read, or -1
+  uint8_t * const coe_buf;		// copy of last good sector read
+  const uint8_t * bsd_buf;		// bad_sector_data buffer
 					// variables for update_rates
   long long a_rate, c_rate, first_size, last_size;
   long long iobuf_ipos;			// last pos read in iobuf, or -1
   long long last_ipos;
-  long t0, t1, ts;			// start, current, last successful
+  long long t0, t1, ts;			// start, current, last successful
   Rational tp;				// cumulated pause_on_error
   int oldlen;
   bool rates_updated, current_slow, prev_slow;
@@ -156,6 +162,13 @@ class Rescuebook : public Mapbook, public Rb_options
   bool first_post;			// first read in current pass
   bool first_read;			// first read overall
 
+  void adjust_copied_size( int & copied_size ) const
+    {
+    if( bsd_buf )
+      for( int i = 0; i + hardbs() <= copied_size; i += hardbs() )
+        if( std::memcmp( iobuf() + i, bsd_buf, hardbs() ) == 0 )
+          { copied_size = i; errno = EIO; break; }
+    }
   void change_chunk_status( const Block & b, const Sblock::Status st );
   void do_pause_on_error();
   bool extend_outfile_size();
@@ -195,8 +208,15 @@ public:
               const char * const iname, const char * const oname,
               const char * const mapname, const int cluster,
               const int hardbs, const bool synchronous );
-  ~Rescuebook() { delete[] voe_buf; }
+  ~Rescuebook() { delete[] coe_buf; if( bsd_buf ) delete[] bsd_buf; }
 
+  bool read_bad_sector_data( const int fd );
   int do_commands( const int ides, const int odes );
   int do_rescue( const int ides, const int odes );
   };
+
+
+const char * const disap_msg = "Input file disappeared";
+
+// defined in main.cc
+void show_final_msg( const char * const msg, const int errcode );
